function [phi_dot_dot, theta_dot_dot, p_CM_dot_dot] = calculate_q_dot_dot_CM(t, theta_dot, fx, fy, tau, u)
%
% Calculates and returns the acceleration of theta and p_CM and the constraint joint forces
% for each link of the snake robot. Theta are the absolute link angles and
% p_CM is the position of the center of mass of the snake.
%

global n l J A D e E  N l_N l_N_tr V N_N_tr V_x_a V_y_a m...
sin_theta_diag cos_theta_diag  M_temp mi_n cn ct lambda1 lambda2 lambda3 teller t_save


%Calculate useful matrixes

Mtot=[n*m+e'*mi_n*sin_theta_diag*sin_theta_diag*e -e'*mi_n*sin_theta_diag*cos_theta_diag*e;...
    -e'*mi_n*sin_theta_diag*cos_theta_diag*e n*m+e'*mi_n*cos_theta_diag*cos_theta_diag*e;];

Mtot_inv=inv(Mtot);

M11=Mtot_inv(1,1); M12=Mtot_inv(1,2); M21=Mtot_inv(2,1); M22=Mtot_inv(2,2);

N1=l*sin_theta_diag*N*mi_n*sin_theta_diag*sin_theta_diag*e+l*cos_theta_diag*N*mi_n*sin_theta_diag*cos_theta_diag*e;
N2=l*sin_theta_diag*N*mi_n*sin_theta_diag*cos_theta_diag*e+l*cos_theta_diag*N*mi_n*cos_theta_diag*cos_theta_diag*e;

K1=(-M11*e'*mi_n*sin_theta_diag*sin_theta_diag+M12*e'*mi_n*sin_theta_diag*cos_theta_diag)*l*N'*sin_theta_diag...
    -(M11*e'*mi_n*sin_theta_diag*cos_theta_diag-M12*e'*mi_n*cos_theta_diag*cos_theta_diag)*l*N'*cos_theta_diag;
 
K2=(-M11*e'*mi_n*sin_theta_diag*sin_theta_diag+M12*e'*mi_n*sin_theta_diag*cos_theta_diag)*l*N'*cos_theta_diag...
    +(M11*e'*mi_n*sin_theta_diag*cos_theta_diag-M12*e'*mi_n*cos_theta_diag*cos_theta_diag)*l*N'*sin_theta_diag;

K3=M11*e'*mi_n*sin_theta_diag*cos_theta_diag*V_x_a-M12*e'*mi_n*cos_theta_diag*cos_theta_diag*V_x_a...
    +M11*e'*mi_n*sin_theta_diag*sin_theta_diag*V_y_a-M12*e'*mi_n*sin_theta_diag*cos_theta_diag*V_y_a;

K4=(-M21*e'*mi_n*sin_theta_diag*sin_theta_diag+M22*e'*mi_n*sin_theta_diag*cos_theta_diag)*l*N'*sin_theta_diag...
    -(M21*e'*mi_n*sin_theta_diag*cos_theta_diag-M22*e'*mi_n*cos_theta_diag*cos_theta_diag)*l*N'*cos_theta_diag;
 

K5=(-M21*e'*mi_n*sin_theta_diag*sin_theta_diag+M22*e'*mi_n*sin_theta_diag*cos_theta_diag)*l*N'*cos_theta_diag...
    +(M21*e'*mi_n*sin_theta_diag*cos_theta_diag-M22*e'*mi_n*cos_theta_diag*cos_theta_diag)*l*N'*sin_theta_diag;

K6=M21*e'*mi_n*sin_theta_diag*cos_theta_diag*V_x_a-M22*e'*mi_n*cos_theta_diag*cos_theta_diag*V_x_a...
    +M21*e'*mi_n*sin_theta_diag*sin_theta_diag*V_y_a-M22*e'*mi_n*sin_theta_diag*cos_theta_diag*V_y_a;

% Calculates the inertia matrix

M=J*eye(n)+m*l^2*sin_theta_diag*V*sin_theta_diag+m*l^2*cos_theta_diag*V*cos_theta_diag+lambda1*eye(n)...
    +l^2*sin_theta_diag*N*mi_n*sin_theta_diag*sin_theta_diag*N'*sin_theta_diag...
    +l^2*sin_theta_diag*N*mi_n*sin_theta_diag*cos_theta_diag*N'*cos_theta_diag...
    +l^2*cos_theta_diag*N*mi_n*sin_theta_diag*cos_theta_diag*N'*sin_theta_diag...
    +l^2*cos_theta_diag*N*mi_n*cos_theta_diag*cos_theta_diag*N'*cos_theta_diag...
    +N1*K1-N2*K4;

% Calculates various other matrices

W = m*l^2*sin_theta_diag*V*cos_theta_diag-m*l^2*cos_theta_diag*V*sin_theta_diag...
    +l^2*sin_theta_diag*N*mi_n*sin_theta_diag*sin_theta_diag*N'*cos_theta_diag...
    -l^2*sin_theta_diag*N*mi_n*sin_theta_diag*cos_theta_diag*N'*sin_theta_diag...
    +l^2*cos_theta_diag*N*mi_n*sin_theta_diag*cos_theta_diag*N'*cos_theta_diag...
    -l^2*cos_theta_diag*N*mi_n*cos_theta_diag*cos_theta_diag*N'*sin_theta_diag...
    +N1*K2-N2*K5;

V_theta=lambda2*eye(n)-l*sin_theta_diag*N*mi_n*sin_theta_diag*cos_theta_diag*V_x_a...
    -l*sin_theta_diag*N*mi_n*sin_theta_diag*sin_theta_diag*V_y_a...
    -l*cos_theta_diag*N*mi_n*cos_theta_diag*cos_theta_diag*V_x_a...
    -l*cos_theta_diag*N*mi_n*sin_theta_diag*cos_theta_diag*V_y_a...
    +N1*K3-N2*K6;
    

% Calculates a vector of squared angular link velocities
theta_dot_sq = zeros(n, 1);
theta_dot_abs = zeros(n, 1);

for i = 1:n
    theta_dot_sq(i, 1) = theta_dot(i, 1)*theta_dot(i, 1);
    theta_dot_abs(i, 1) = theta_dot(i, 1)*abs(theta_dot(i, 1));
end

% Constructs the inertia matrix of the system

M_inv=inv(M);

% Calculates a matrix related to Coriolis and centripetal forces
W_th=W*theta_dot_sq+V_theta*theta_dot+lambda3*eye(n)*theta_dot_abs...
    +(N1*M11*e'-N2*M21*e'-l*sin_theta_diag*N)*fx+(N1*M12*e'-N2*M22*e'+l*cos_theta_diag*N)*fy;

% Calculates the theta_dot_dot
theta_dot_dot =M_inv*(D'*u-W_th);


px_dot_dot=K1*theta_dot_dot+K2*theta_dot_sq+K3*theta_dot+M11*e'*fx+M12*e'*fy;
py_dot_dot=K4*theta_dot_dot+K5*theta_dot_sq+K6*theta_dot+M21*e'*fx+M22*e'*fy;

p_CM_dot_dot=[px_dot_dot;py_dot_dot];

teller=[teller; M];
t_save=[t_save;t];

% Calculates the joint angle acceleration
phi_dot_dot=D*theta_dot_dot;