%
% Visualizes the snake robot motion in a graph window.
%

global save_to_AVI AVI_file_name AVI_fps AVI_speedfactor AVI_include_axes

% Initializes AVI file creation
aviobj = [];
if save_to_AVI
    % Closes all open AVI files
    clear mex
    
    % Creates the new AVI file
    %aviobj = avifile(AVI_file_name, 'fps', AVI_fps, 'compression', 'Indeo5', 'quality', 100);
    aviobj = avifile(AVI_file_name, 'fps', AVI_fps, 'quality', 100); % Cinepak gir best kvalitet.
end

% Draws the initial position of the snake robot
drawSnakeRobot(t(1), theta(1, :)', p_CM(1, :)');

% The time of the previously drawn sample
previous_draw_time = 0;

% The step time to be used for visualizing the data
step = 0.02;
if save_to_AVI
    step = 1/AVI_fps/3;
end

% Vector of data indices that are chosen so that the time interval between
% each data corresponding to each index is equal to the specified time step
index_vector = [1];

% The simulation time corresponding to the previous index stored in the
% index vector
previous_time = 0;

i = 2;
while i < size(theta, 1)
    % Finds the index of the next position to be drawn
    while i < size(theta, 1) && t(i) <= previous_time + step
        i = i+1;
    end
    % Stores the next index
    index_vector(end+1, 1) = i;
    previous_time = t(i);
end

% Increases the index_counter so that visualization starts from the
% specified start time
index_counter = 1;
starttime = 0;
currenttime = 0;
while currenttime < starttime
    % Increases the index counter
    index_counter = index_counter + 1;
    % Extracts the next index
    next_index = index_vector(index_counter, 1);
    currenttime = t(next_index);
end

% Visualizes all data corresponding to each index in the index vector
previous_AVI_file_update = 0;
while index_counter < size(index_vector, 1)
    % Extracts the next index
    next_index = index_vector(index_counter, 1);
    % Draws the next position
    drawSnakeRobot(t(next_index), theta(next_index, :)', p_CM(next_index, :)');
    % Gives Matlab time to show the drawn graph
    pause(0.001);
    %pause(0.1)
    % Increases the index counter
    index_counter = index_counter + 1;
    
    % Saves to AVI file
    if save_to_AVI && (t(next_index) - previous_AVI_file_update) > (1/AVI_fps)*AVI_speedfactor
        disp(['Added a frame at time ' num2str(t(next_index)) '. Frame interval: ' num2str(t(next_index) - previous_AVI_file_update)])
        frame = [];
        if AVI_include_axes
            frame = getframe(gcf);
        else
            frame = getframe(gca);
        end
        
        aviobj = addframe(aviobj, frame);
        previous_AVI_file_update = t(next_index);
    end
    
end

% Closes the AVI file
if save_to_AVI
    aviobj = close(aviobj);
end
