function v_dot = calculate_v_dot_eel(t, v)
%
% Calculates and returns v_dot.
%
% v = [theta ; p_CM ; theta_dot ; p_CM_dot]
%

global n D sin_theta_diag cos_theta_diag friction_model ...
    previous_draw_time visualize_motion timestep ...
    store_sim_data sim_data_save_interval previous_simdata_save ...
     t_sim z_sim phi phi_ref phi_ref_sim phi_sim z_sim_dot fxt_sim fyt_sim...
     v_dot_dot_old qw u_sim delta_sim tail_model  phi_dot_sim fx_sim fy_sim


% Extracts the states from the state vector
theta               = v(1:n, 1);
p_CM                = v(n+1:n+2, 1);
theta_dot           = v(n+3:2*n+2, 1);
p_CM_dot            = v(2*n+3:2*n+4, 1);

% Calculates diagonal matrices that are used in the proceeding calculations
sin_theta_diag = calculate_sin_theta_diag(theta);
cos_theta_diag = calculate_cos_theta_diag(theta);

% Calculates the joint angles
phi = D*theta;
% Calculates the joint angle velocities
phi_dot = D*theta_dot;

% Calculates the position of the links
z = calculate_z(theta, [], p_CM);

% Calculates the linear velocities of the links
z_dot = calculate_z_dot(theta_dot, [], p_CM_dot);

% Calculates translational friction forces.
fx = zeros(n, 1);
fy = zeros(n, 1);
if strcmp(friction_model, 'MORISON')
    [fx, fy] = calculate_f_drag(theta_dot, z_dot);
elseif strcmp(friction_model, 'VISCOUS')
    [fx, fy, fx_debug] = calculate_f_viscous(t, theta, theta_dot, p_CM_dot, z_dot);
end

  

% Calculates rotational friction forces (friction torques).
tau = zeros(n, 1);
%tau = calculate_tau_coulomb(theta_dot, z_dot);

% Calculates control input according to lateral undulation without any
% directional control.
  %[u, phi_ref] = calculate_u_eel_undulation(t, phi, phi_dot, theta, p_CM, p_CM_dot);
  [u, phi_ref] = calculate_u_eel_undulation(t, phi, phi_dot, theta, p_CM, p_CM_dot);
  %[u, phi_ref] = calculate_u_lateral_undulation(t, phi, phi_dot, theta, p_CM, p_CM_dot);

%Calculate the foil cords
[c_d, h_d , link, c_c, h_c ]=cords();

 %Calculates the tale force
 
 if strcmp(tail_model, 'foil')
    [fxt, fyt, tau_t, delta]=StiffTail(z, z_dot, theta, p_CM, p_CM_dot, theta_dot, c_d, h_d , link, c_c, h_c);
elseif strcmp(tail_model, 'NO')
    fxt=zeros(n,1);
    fyt=zeros(n,1);
    tau_t=zeros(n,1);
    delta=0;
end


  % Calculates the acceleration of theta and p where p is the position of the CM
[phi_dot_dot, theta_dot_dot, p_CM_dot_dot] = calculate_q_dot_dot_CM(t, theta_dot, fx, fy, fxt, fyt, tau, tau_t, u);

% Constructs the derivative of the state vector
v_dot = [theta_dot ; p_CM_dot ; theta_dot_dot ; p_CM_dot_dot];

%v_dot_dot_old=v_dot;

% Visualizes the snake robot
if visualize_motion
    % Draws the snake if the specified time step has passed
    if t >= previous_draw_time + timestep*10
        % Updates the time of the previously drawn position
        previous_draw_time = t;
        % Draws the next position
        drawSnakeRobot(t, theta, p_CM);
        % Gives Matlab time to show the drawn graph
        pause(timestep);
        %pause(0.2);
    end
end

% Stores simulation data for debug purposes.
if store_sim_data
    % Clears stored simulation data.
    if t <= 0
        previous_simdata_save = 0;
        t_sim = [];
        z_sim = [];
        z_sim_dot = [];
        fxt_sim=[];
        fyt_sim=[];
        u_sim=[];
        delta_sim=[];
         phi_dot_sim=[];
         fx_sim=[];
         fy_sim=[];
    end
    
    % Stores simulation data for debug purposes.
    if t >= previous_simdata_save + sim_data_save_interval
        previous_simdata_save = t;
        t_sim(end+1, 1) = t;
        z_sim(end+1, :) = z';
        z_sim_dot(end+1, :) = z_dot';
        phi_sim(end+1, :)=phi;
        phi_dot_sim(end+1, :)=phi_dot;
        phi_ref_sim(end+1, :)=phi_ref;
         fxt_sim(end+1,:)=fxt';
        fyt_sim(end+1,:)=fyt';
        fx_sim(end+1,:)=fx';
        fy_sim(end+1,:)=fy';
        u_sim(end+1,:)=u';
        delta_sim(end+1,:)=delta;

    end
end
