function drawSnakeRobot(t, theta, p)
%
% Draws the snake robot in a graph window.
%

global n l fignumber sin_theta_diag cos_theta_diag ...
    visualizeCMposition l_tot

% Sets figure properties
window = figure(fignumber);

% Indicates that the erasing and redrawing of a rendering occurs off-screen
% to avoid flickering
set(window, 'DoubleBuffer', 'on');
clf

% Sets the background color.
set(window, 'color', 'white');
hold on;
%grid on;

axis('equal');
xlabel('X [m]');
ylabel('Y [m]');
% Rounds the time to two decimals precision
%t = round(t*100)/100;
title(['t = ' num2str(t)]);

% Calculates the position of the links.
sin_theta_diag = calculate_sin_theta_diag(theta);
cos_theta_diag = calculate_cos_theta_diag(theta);
z = calculate_z(theta, [], p);
% Partitions the position vector.
x = z(1:n);
y = z(n+1:2*n);

% Draws the snake robot
for j = 1:n
    % Calculates the global frame coordinates of the beginning of the next link.
    startx = x(j, 1) - l(j,j)*cos(theta(j, 1));
    starty = y(j, 1) - l(j,j)*sin(theta(j, 1));
    % Calculates the global frame coordinates of the end of the next link.
    endx = x(j, 1) + l(j,j)*cos(theta(j, 1));
    endy = y(j, 1) + l(j,j)*sin(theta(j, 1));
    
    % Draws the next link as a line.
    plot([startx ; endx], [starty ; endy], 'bo-', 'linewidth', 1.3)
    % Draws a point at the CM of each link.
    %plot(x, y, 'ro')
    axis on
    grid on
end

% Draws the position of the CM (center of mass)
if visualizeCMposition
    plot(p(1, 1), p(2, 1), 'r*')
end

% Close-up rectangular view
%axis([p(1,1)-n*l-0.3, p(1,1)+n*l+0.3, p(2,1)-n*l-0.1, p(2,1)+n*l+0.1]);
% Close-up flat view
%axis([p(1,1)-n*l-0.1, p(1,1)+n*l+0.1, p(2,1)-6*l, p(2,1)+6*l]);
% Zoomed out view
%axis([-0.7, 3.5, -0.4, 0.4]);
% Zoomed out view
%axis([-0.7, 1.5, -0.4, 0.4]);

axis([p(1,1)-l_tot-3, p(1,1)+l_tot+3, -3, 3]);

hold off
