% Starts the snake robot simulation.
%
clear all
clc

global visualize_motion previous_draw_time phi phi_ref phi_ref_sim phi_sim...
    z_sim_dot t_sim l_tot fxt_sim fyt_sim  phi_dot_sim fx_sim fy_sim


% The stop time for the simulation.
start_time = 0;
stop_time = 60;

% Initializesmodel parameters.
initModelParameters;

% Initializes controller parameters.
initControllerParameters;

disp('Starting simulation...')

% Matlab function used to measure how long the simulation takes
tic

% Constructs the initial values for the state vector.
v0 = [theta0 ; p_CM0 ; theta0_dot ; p_CM0_dot];

% Starts the snake robot visualization
if 1*visualize_motion
    % The time of the previously drawn sample
    previous_draw_time = 0;

    % Draws the initial position of the snake robot
    drawSnakeRobot(0, theta0, p_CM0);
    
    % Waits for the figure window to display
    pause(0.1);
end

options = odeset('Events', 'off', 'RelTol', 1e-4, 'AbsTol', 1e-4);
%[t, v] = ode23tb('calculate_v_dot', [start_time stop_time], v0, options);

[t, v] = ode23tb('calculate_v_dot_eel', [start_time stop_time], v0, options);


disp('Simulation complete...')

% Displays how long the simulation took to complete
toc

disp(['Simulated time: ' num2str(t(end, 1)) 's'])

% Extracts the states from the state vector.
theta       = v(:, 1:n);
p_CM        = v(:, n+1:n+2);
theta_dot   = v(:, n+3:2*n+2);
p_CM_dot    = v(:, 2*n+3:2*n+4);

if 1
    % Calculates speed
    disp('Speed:')
    speed = sqrt(p_CM(end, 1)^2 + p_CM(end, 2)^2)/stop_time
    
    figure(23)
    plot(t, p_CM_dot(:,1))
    grid on
end

% Font size in figures.
label_font_size = 28;
axis_font_size = 18;
%figpos = [972   548   750   322];
%figpos = [980   401   752   365]; % ICRA 2011
figpos = [972   483   603   387]; % Avhandling

% Plots the CM position.
if 0

window = figure(7);
set(window, 'color', 'white');
plot(p_CM(:, 1), p_CM(:, 2), 'k--', 'linewidth', 1)
%plot(p_CM(:, 1), p_CM(:, 2), 'b-')
hold on
%grid on
%plot(p_bar_sim(:, 1), p_bar_sim(:, 2), 'g:')
xlabel('p_x [m]','FontSize',label_font_size);
ylabel('p_y [m]','FontSize',label_font_size);
set(gca,'FontSize',axis_font_size)
axis([-1 12 -3 12])
%set(gcf,'Position',figpos)
axis('equal')
%axis([-0.1 3.1 -0.2 2.3])
%xlim([-0.5 3])
% Draws the snake robot.
%plot_tid_vector = [55 125 235]';
plot_tid_vector = [10 29 45]';
link_color = 'b-';
for i = 1:size(plot_tid_vector, 1)
    plot_tid = plot_tid_vector(i, 1);
    plot_index = 1;
    while t(plot_index, 1) < plot_tid
        plot_index = plot_index + 1;
    end
    drawSnakeRobot_plotpurposes(theta(plot_index, :)', p_CM(plot_index, :)', link_color);
end


hold off

h = legend('Path of CM of robot', 'Snake Robot', 'location', 'north');
set(h,'fontsize',12,'interpreter','latex');
legend('boxoff')
end

% Plots the CM velocity.
if 1
    window = figure(20);
    set(window, 'color', 'white');
    plot(t, p_CM_dot(:, 1)*100, 'b')
    grid on
    xlabel('Time [s]','FontSize',label_font_size);
    ylabel('$\dot{p}_x$ [cm/s]','FontSize',label_font_size,'interpreter','latex');
    set(gca,'FontSize',axis_font_size)
    set(gcf,'Position',figpos)
    set(gca,'XTick',[0 20 40 60 80]);
    %ylim([-5 15])
    
    window = figure(21);
    set(window, 'color', 'white');
    plot(t, p_CM_dot(:, 2)*100, 'b')
    grid on
    xlabel('Time [s]','FontSize',label_font_size);
    ylabel('$\dot{p}_y$ [cm/s]','FontSize',label_font_size,'interpreter','latex');
    set(gca,'FontSize',axis_font_size)
    set(gcf,'Position',figpos)
    set(gca,'XTick',[0 20 40 60 80]);
    %ylim([-5 15])
end


% Plots the CM velocity.
if 1
    window = figure(22);
    set(window, 'color', 'white');
    plot(t, theta)
    grid on
    xlabel('Time [s]','FontSize',label_font_size);
    ylabel('$\theta$ [rad]','FontSize',label_font_size,'interpreter','latex');
    set(gca,'FontSize',axis_font_size)
    set(gcf,'Position',figpos)
    set(gca,'XTick',[0 20 40 60 80]);
    %ylim([-5 15])
    
    
end

% Plots the 1st joint error angle.
if 1
    window = figure(25);
    set(window, 'color', 'white');
    plot(t_sim,phi_ref_sim(:,1),'r')
    hold on
    plot(t_sim,phi_sim(:,1),'k--')
    grid on
    xlabel('Time [s]','FontSize',label_font_size);
    ylabel('$\phi$ [rad]','FontSize',label_font_size,'interpreter','latex');
    legend('desierd','real');
    set(gca,'FontSize',axis_font_size)
    set(gcf,'Position',figpos)
    %set(gca,'XTick',[0 20 40 60 80]);
    %ylim([-5 15])
end


% Plots the 1st joint error angle.
if 1
    window = figure(26);
    set(window, 'color', 'white');
    plot(t_sim,u_sim(:,9),'r')
    grid on
    xlabel('Time [s]','FontSize',label_font_size);
    ylabel('u [Nm]','FontSize',label_font_size,'interpreter','latex');
    set(gca,'FontSize',axis_font_size)
    set(gcf,'Position',figpos)
    %set(gca,'XTick',[0 20 40 60 80]);
    %ylim([-5 15])
end



% Calculate the work/m
sp=ones((n-1),1);
P=abs(phi_dot_sim.*u_sim*sp);
Q = trapz(t_sim,P);

disp('Work/m:')
W_m=Q/(sqrt(p_CM(end, 1)^2 + p_CM(end, 2)^2))
